@@ -1,15 +1,22 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-import socket
-import threading
-import multiprocessing
 from typing import Callable
+from threading import Thread
+from multiprocessing import Process
+import socket
 
-from ..logger import Logger
+from ..classes.logger import Logger
 
 
 class ServerSocket():
-    def __init__(self, *, protocol: str, bind: tuple[str, int], group: str | None = None, on_recv: Callable):
+    def __init__(
+            self,
+            *,
+            protocol: str,
+            bind: tuple[str, int],
+            group: str | None = None,
+            on_recv: Callable[[bytes, tuple[str, int], Callable[[bytes], int]], None],
+        ):
         """服务端套接字
 
         在新线程中创建 TCP/UDP/MULTICAST 协议的服务端套接字，接收客户端的
@@ -28,9 +35,9 @@ class ServerSocket():
 
         Args:
             protocol (str): 协议
-            bind (tuple[str, int]): 绑定的地址, 注: 当多网卡, 且 ip 为 "0.0.0.0" 时, 有可能接收不到数据.
+            bind (tuple[str, int]): 绑定的地址, 端口为 `0` 时随机分配端口, 注: 当多网卡, 且 ip 为 "0.0.0.0" 时, 有可能接收不到数据.
             group (tuple[str, int] | None, optional): 组播地址, 仅在协议类型为 "MULTICAST" 时有效. 默认为 None.
-            on_recv (Callable, optional): 接收到数据时的回调函数, 参数为 (data: bytes, client_name: str, send_back: Callable). 默认为 None.
+            on_recv (Callable, optional): 接收到数据时的回调函数, 参数为 (data: bytes, client_name: str, send_back: Callable[[bytes], int]). 默认为 None.
 
         Raises:
             ValueError: 无效的协议类型, 应为 [TCP, UDP, MULTICAST]
@@ -41,6 +48,7 @@ class ServerSocket():
         Examples:
         """
         self.__active = False
+        self.__socked = False
 
         if protocol not in ['TCP', 'UDP', 'MULTICAST']:
             raise ValueError(f'ServerSocket 无效的协议类型 "{protocol}"')
@@ -50,8 +58,10 @@ class ServerSocket():
             raise ValueError(f'ServerSocket 协议类型为 "{protocol}" 时请勿设置 group 参数')
         if not bind[0]:
             raise ValueError(f'ServerSocket 绑定地址不能为空')
-        if bind[1] < 1 or bind[1] > 65535:
+        if bind[1] < 0 or bind[1] > 65535:
             raise ValueError(f'ServerSocket 无效的端口号 "{bind[1]}"')
+        if not callable(on_recv):
+            raise ValueError(f'ServerSocket on_recv 参数必须为可调用对象')
 
         self.logger = Logger()
         self.protocol = protocol
@@ -60,7 +70,7 @@ class ServerSocket():
         self.on_recv = on_recv
         self.sock: socket.socket | None = None
         self.tcp_sub_socks: list[socket.socket] = []
-        self.thread: threading.Thread | None = None
+        self.thread: Thread | Process | None = None
 
     def __str__(self) -> str:
         if self.protocol == 'MULTICAST':
@@ -68,28 +78,37 @@ class ServerSocket():
         return f'ServerSocket({self.protocol}, bind {self.bind[0]}:{self.bind[1]})'
 
     def __del__(self) -> None:
-        if self.is_active():
-            self.close()
+        self.close()
 
     def __create_socket(self) -> None:
-        match self.protocol:
-            case 'TCP':
-                self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-                self.sock.bind(self.bind)
-                self.sock.listen(10)
-            case 'UDP':
-                self.sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
-                self.sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
-                self.sock.bind(self.bind)
-            case 'MULTICAST':
-                self.sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
-                self.sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
-                self.sock.bind(self.bind)
-                self.sock.setsockopt(
-                    socket.IPPROTO_IP,
-                    socket.IP_ADD_MEMBERSHIP,
-                    socket.inet_aton(self.group) + socket.inet_aton(self.bind[0]),
-                )
+        if self.__socked:
+            return True
+
+        try:
+            match self.protocol:
+                case 'TCP':
+                    self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+                    self.sock.bind(self.bind)
+                    self.sock.listen(10)
+                case 'UDP':
+                    self.sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
+                    self.sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
+                    self.sock.bind(self.bind)
+                case 'MULTICAST':
+                    self.sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
+                    self.sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
+                    self.sock.bind(self.bind)
+                    self.sock.setsockopt(
+                        socket.IPPROTO_IP,
+                        socket.IP_ADD_MEMBERSHIP,
+                        socket.inet_aton(self.group) + socket.inet_aton(self.bind[0]),
+                    )
+            self.bind = self.sock.getsockname()
+            self.__socked = True
+        except Exception as e:
+            self.logger.error(f'{self} 创建失败: \n{e}')
+
+        return self.__socked
 
     def __send_back(self, client_addr: tuple[str, int], client_sock: socket.socket | None = None) -> Callable:
         def send_back(data: bytes):
@@ -109,11 +128,7 @@ class ServerSocket():
                     break
 
                 self.logger.debug(f'{self} TCP 子线程 {client_addr} 接收到数据: {data}')
-                self.on_recv and self.on_recv(
-                    data=data,
-                    client_addr=client_addr,
-                    send_back=self.__send_back(client_addr, client_sock),
-                )
+                self.on_recv(data, client_addr, self.__send_back(client_addr, client_sock))
             except ConnectionResetError:
                 self.logger.debug(f'{self} TCP 子线程 {client_addr} 连接已重置')
                 break
@@ -137,21 +152,29 @@ class ServerSocket():
                         client_sock, client_addr = self.sock.accept()
                         self.logger.debug(f'{self} 与 {client_addr} 建立 TCP 连接')
                         self.tcp_sub_socks.append(client_sock)
-                        threading.Thread(target=self.__tcp_sub_thread, args=(client_sock, client_addr), daemon=True).start()
+                        Thread(target=self.__tcp_sub_thread, args=(client_sock, client_addr), daemon=True).start()
                 else:
                     data, client_addr = self.sock.recvfrom(1024)
 
                     self.logger.debug(f'{self} 收到 {client_addr} 的数据: {data}')
-                    self.on_recv and self.on_recv(
-                        data=data,
-                        client_addr=client_addr,
-                        send_back=self.__send_back(client_addr),
-                    )
+                    self.on_recv(data, client_addr, self.__send_back(client_addr))
             except Exception as e:
                 if self.is_active():
                     self.logger.error(f'{self} 主线程异常 : \n{e}')
                     break
 
+    def getpeername(self) -> tuple[str | None, int | None]:
+        """返回套接字连接到的远程地址。"""
+        if self.__create_socket():
+            return self.sock.getpeername()
+        return (None, None)
+
+    def getsockname(self) -> tuple[str | None, int | None]:
+        """返回套接字本身的地址。"""
+        if self.__create_socket():
+            return self.sock.getsockname()
+        return (None, None)
+
     def send(self, data: bytes, client_addr: tuple[str, int]) -> int:
         """向指定客户端发送数据
 
@@ -162,15 +185,14 @@ class ServerSocket():
         Returns:
             int: 实际发送的字节数, 为 -1 表示 socket 未建立或已关闭
         """
-        if not self.is_active():
-            return -1
-
-        if self.protocol == 'TCP':
-            for client_sock in self.tcp_sub_socks:
-                if client_sock.getpeername() == client_addr:
-                    return client_sock.sendall(data)
-            return 0
-        return self.sock.sendto(data, client_addr)
+        if self.__create_socket():
+            if self.protocol == 'TCP':
+                for client_sock in self.tcp_sub_socks:
+                    if client_sock.getpeername() == client_addr:
+                        return client_sock.sendall(data)
+                return 0
+            return self.sock.sendto(data, client_addr)
+        return -1
 
     def start(self, is_process: bool = False) -> bool:
         """启动服务端
@@ -183,18 +205,15 @@ class ServerSocket():
         Returns:
             bool: 是否启动成功
         """
-        try:
-            self.__create_socket()
-        except Exception as e:
-            self.logger.error(f'{self} 创建失败: \n{e}')
-            return False
+        if not self.thread and self.__create_socket():
+            if is_process:
+                self.thread = Process(target=self.__main_thread, daemon=True)
+            else:
+                self.thread = Thread(target=self.__main_thread, daemon=True)
+            self.thread.start()
+            return True
 
-        if is_process:
-            self.thread = multiprocessing.Process(target=self.__main_thread, daemon=True)
-        else:
-            self.thread = threading.Thread(target=self.__main_thread, daemon=True)
-        self.thread.start()
-        return True
+        return self.is_active()
 
     def close(self) -> bool:
         """关闭服务端
@@ -202,29 +221,33 @@ class ServerSocket():
         Returns:
             bool: 是否关闭成功
         """
-        try:
-            self.__active = False
-            if self.protocol == 'TCP':
-                for client_sock in self.tcp_sub_socks:
-                    client_sock.shutdown(socket.SHUT_RDWR)
-                    client_sock.close()
-                self.tcp_sub_socks.clear()
+        if self.__socked:
             try:
-                # windows 平台 TCP 无需 shutdown (此时 shutdown 会导致 10057 错误)
-                # linux 平台非 TCP shutdown 会报 107 错误
-                self.sock.shutdown(socket.SHUT_RDWR)
-            except OSError as e:
-                if e.errno != 107 and e.errno != 10057:
-                    raise e
-            self.sock.close()
-            if isinstance(self.thread, multiprocessing.Process):
-                self.thread.terminate()
-            else:
-                self.thread.join()
-        except Exception as e:
-            self.logger.error(f'{self} 关闭失败: \n{e}')
-            return False
-        return True
+                self.__active = False
+                if self.protocol == 'TCP':
+                    for client_sock in self.tcp_sub_socks:
+                        client_sock.shutdown(socket.SHUT_RDWR)
+                        client_sock.close()
+                    self.tcp_sub_socks.clear()
+                try:
+                    # windows 平台 TCP 无需 shutdown (此时 shutdown 会导致 10057 错误)
+                    # linux 平台非 TCP shutdown 会报 107 错误
+                    self.sock.shutdown(socket.SHUT_RDWR)
+                except OSError as e:
+                    if e.errno != 107 and e.errno != 10057:
+                        raise e
+                self.sock.close()
+                if isinstance(self.thread, Process):
+                    self.thread.terminate()
+                else:
+                    self.thread.join()
+
+                self.__socked = False
+                self.logger.debug(f'{self} 已关闭')
+            except Exception as e:
+                self.logger.error(f'{self} 关闭失败: \n{e}')
+
+        return self.__socked
 
     def is_active(self) -> bool:
         """返回服务端是否处于活动状态
@@ -232,4 +255,4 @@ class ServerSocket():
         Returns:
             bool: 是否处于活动状态
         """
-        return self.__active
+        return self.__socked and self.__active
